C---------------------------------------------------------------
C To calculate the axis-angle pair relating two cubic lattices
C  from an input which consists of a pair of vectors from one
C  crystal, and another pair from the second crystal, together
C  with an angle between the two sets. All vectors are coplanar.
C All 24 symmetry-related axis-angle pairs are computed.
C
      SUBROUTINE MAP_CRYSTAL_PAIR(H,K,L,H1,K1,L1,U1,V1,W1,U,V,W,
     &BEC,R,AINVR,PI,X,Y,Z,N1,Q1,Q2,Q3,THET,PHI)
      IMPLICIT NONE
      REAL H,K,L,H1,K1,L1,U1,V1,W1,U,V,W,PI,PHI
      REAL H2,K2,L2,Q,M,P,A1,A2,A3,A4,A5,A6,A7,A8,A9
      REAL C2,C3,C4,C5,C,C10,H3,K3,L3,H4,K4,L4
      REAL X,Y,Z,N,N1,XXX,YYY,ZZZ,NNNN1
      REAL BEC(24,9),B(9),AA(9),R(9),AINVR(9)
      REAL Q1(23),Q2(23),Q3(23),THET(23)
      INTEGER I,I20,I21
C
        PHI=PHI*2.0*PI/360.0
C Normalize the components of each input vector, i.e. redefine
C  in an orthonormal coordinates
        CALL MAP_UTIL_NORM(H,K,L)
        CALL MAP_UTIL_NORM(H1,K1,L1)
        CALL MAP_UTIL_NORM(U1,V1,W1)
        CALL MAP_UTIL_NORM(U,V,W)
C For crystal 1, calculate a third orthogonal vector using a vector
C   cross product
        H2=K*L1-L*K1
        K2=L*H1-H*L1
        L2=H*K1-K*H1
        CALL MAP_UTIL_NORM(H2,K2,L2)
C Calculate angle between the pair of input vectors of crystal 1
        Q=H*H1+K*K1+L*L1
8       Q=ACOS(Q)
C Calculate the angle between the pair of input vectors of crystal 2
        M=U*U1+V*V1+W*W1
10      M=ACOS(M)
C Calculate a further angle P
      P=M-PHI
C
        A1=H2
        A2=K2
        A3=L2
        A4=H1
        A5=K1
        A6=L1
        A7=H
        A8=K
        A9=L
C
        C2=COS(PHI)
        C3=COS(PHI+Q)
        C4=COS(P)
        C5=COS(Q-P)
C
      B(1)=A5*A9-A6*A8
      B(2)=A6*A7-A4*A9
      B(3)=A4*A8-A7*A5
      B(4)=A8*A3-A9*A2
      B(5)=A9*A1-A7*A3
      B(6)=A7*A2-A8*A1
      B(7)=A2*A6-A3*A5
      B(8)=A3*A4-A1*A6
      B(9)=A1*A5-A2*A4
      C=A1*B(1)+A2*B(2)+A3*B(3)
C
      DO 11 I=1,9
      AA(I)=B(I)/C
11    CONTINUE
      H3=AA(4)*C2+AA(7)*C3
      K3=AA(5)*C2+AA(8)*C3
      L3=AA(6)*C2+AA(9)*C3
      H4=AA(4)*C4+AA(7)*C5
      K4=AA(5)*C4+AA(8)*C5
      L4=AA(6)*C4+AA(9)*C5
      CALL MAP_UTIL_NORM(H3,K3,L3)
      CALL MAP_UTIL_NORM(H4,K4,L4)
      X=(K4-V1)*(L3-W)-(L4-W1)*(K3-V)
      Y=(L4-W1)*(H3-U)-(H4-U1)*(L3-W)
      Z=(H4-U1)*(K3-V)-(K4-V1)*(H3-U)
      C10=(H4+U1)*(H3-U)+(K4+V1)*(K3-V)+(L3-W)*(L4+W1)
      X=X/C10
      Y=Y/C10
      Z=Z/C10
      M=SQRT(X*X+Y*Y+Z*Z)
      N=2.0*ATAN(M)
      N1=N*360.0/(2.0*PI)
      CALL MAP_UTIL_NORM(X,Y,Z)
C
C
      CALL MAP_UTIL_ROT(X,Y,Z,N,R)
C 
        XXX=X
        YYY=Y
        ZZZ=Z
        NNNN1=N1
C
C Initialize matrix BEC
        DO 450 I20=1,24
          DO 451 I21=1,9
           BEC(I20,I21)=0.0
451       CONTINUE
450     CONTINUE
C Calculate the 24 symmetry related rotation matrices
      CALL MAP_CRYSTAL_ORIENT(BEC)
C      
C
C Calculate the inverse of the rotation matrix
      CALL MAP_UTIL_INVERS(AINVR,R)
      CALL MAP_CRYSTAL_ROTAT(XXX,YYY,ZZZ,NNNN1,BEC,Q1,Q2,Q3,THET)
C
C
        RETURN
        END
